#
# mkindex.rb
#
#   Copyright (c) 2001 Minero Aoki <aamine@cd.xdsl.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU General Public License version 2 or later.
#

require 'librefe'


def main
  t = nil
  File.open( ARGV[0] || 'manual.rd' ) do |f|
    t = create_index(f)
  end
  File.open( Refe::INDEX_FILE, 'w' ) do |f|
    Marshal.dump t, f
  end
end

def create_index( f )
  table = {}
  mt = st = nil
  klass = buf = s = bc = nil
  isalias = false

  f.each do |line|
    unless /\A: / === line then
      isalias = false
      bc = true

      case line
      when /\A= ([A-Z]\w+(?:::[A-Z]\w+)*)\s*\z/
        klass = $1
        mt = (table[klass + '#'] ||= {})
        st = (table[klass + '.'] ||= {})
      when /\A= /
        klass = nil
      when /\A=+ .*饹᥽å/e,
           /\A=+ ⥸塼ؿ/e,
           /\A=+ ⥸塼°/e,
           /\A=+ ⥸塼᥽å/e
        s = true
      when /\A=+ ᥽å/e,
           /\A=+ ץ饤١ȥ᥽å/e
        s = false
      when /\A=+/
        ;
      else
        buf << line if buf
        bc = false
      end
      if bc then
        bufclear buf
        buf = nil
      end

    else   # method define
      if klass and n = getmname(line) then
        buf = register_method( n, line, buf, isalias, (s ? st : mt) )
        isalias = true
      else
        bufclear buf
        buf = nil
        isalis = false
      end
    end
  end

  Refe.new( table )
end

def register_method( method, line, buf, isalias, table )
  if t = table[method] then
    t << line
    buf = t
  elsif isalias then
    buf << line
    table[method] = buf
  else
    bufclear buf
    table[method] = buf = line
  end
  buf
end

METHOD_RE = [
  [/\A:\s*(\w+[?!]?)\s*(?:[\(\{]|\z)/    , ''   ],# name(arg), name { }, name
 #[/\A:\s*(\w+[?!]?)\s*\z/               , ''   ],# name
 #[/\A:\s*(\w+[?!]?)\s*\{/               , ''   ],#
  [/\A:\s*(\w+)\s*\=/                    , '='  ],# name =
  [/\A:\s*\w+\[.*\]=/                    , '[]='],# self[key]=
  [/\A:\s*\w+\[.*\]/                     , '[]' ],# self[key]
  [/\A:\s*self\s*([+-<>=~*^&|%]+)\s*\w+/ , ''   ],# self + other
  [/\A:\s*([+\-~])\s*\w+/                , ''   ] # ~ self
]

def getmname( line )
  i = _getmname(line)
  printf "%15s %s\n", i.inspect, line.strip if $DEBUG and not i
  i
end

def _getmname( line )
  METHOD_RE.each do |re,s|
    if m = re.match(line) then
      return case s.size
             when 0 then m[1]
             when 1 then m[1] + s
             else        s
             end
    end
  end
  nil
end

def bufclear( str )
  return unless str
  str.gsub! /\(\(-.*-\)\)/, ''
  str.gsub! /\(\([{|'<]|[}|'>]\)\)/, ''
end


main
